#include <stdio.h>
#include <gl.h>
#include <device.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/dir.h>
#include <audio.h>

#define	BS			30
#define	HIGHLIGHT		(64|(0<<8)|(64<<16))
#define	LOWLIGHT		0x000000
#define	BARWIDTH		22
#define AUDIO_MAX_SIZE		15000
#define AUDIO_BUF_SIZE		10000
#define CHUNK_SIZE		512
#define	CONFIGURE		1
#define DURATION		25
#define MAXDURATION		(DURATION<<1)
#define ORDER			64
#define	AUDIO_IN		2
#define	AUDIO_OUT		4
#define PI2			6.283185307179586476925287
#define SWAP(a,b) tempr=(a); (a)=(b); (b) = tempr
#define PWRMAX			100.0
#define	BIGEST			15000.00
#define	NUMBER			(BIGEST / 256.0)
#define	LM			LEFTMOUSE	
#define	MM			MIDDLEMOUSE	
#define	RM			RIGHTMOUSE	
#define	BUTTONDOWN		(getbutton(LM)|getbutton(MM)|getbutton(RM))

typedef struct { float  r; float  i; } complex;

long		ip_audio_open(long);
void		ip_audio_close(long);
void		NRICfft(float *,unsigned int,int);
float		spectrum(short *,int,float *,int,float);
void		ip_audio_reopen(void);
void		ge_draw_basic_button(int,int,int,int);
void		update_amp(void);
void		get_sample(void);
void		parse_it(void);
void		draw_panel(void);
void		draw_gains(void);
void		ip_font(void);
void		draw_icons(void);
void		draw_irate(void);
void		draw_orate(void);
void		wait_4_user(void);
void		setparams(void);
void		set_autogain(long);

extern		unsigned short	raster_f[];
extern		Fontchar        chars[256];

ALconfig	audio_config;
ALport		in_port		= 0;
ALport		out_port	= 0;

short		ab[2*AUDIO_MAX_SIZE];
complex		tmpc[8192];
int		idur		= 0;
float		maxpwr		= PWRMAX;
float		pwr		= 1.0;
float		tpwr		= 0.0;
float		tmaxpwr		= 0.0;
float		lmaxpwr		= -1;
float		lpwr		= -1;
float		amp[MAXDURATION][ORDER];
int		screen_x_size;
int		screen_y_size;
long		llx,lly;
long		window_id;
int		nframe		= 0;
long		tvbuf[]		= { AL_INPUT_RATE,0,
				    AL_INPUT_SOURCE,0,
				    AL_LEFT_INPUT_ATTEN,0,
				    AL_RIGHT_INPUT_ATTEN,0,
				    AL_OUTPUT_RATE,0,
				    AL_LEFT_SPEAKER_GAIN,0,
				    AL_RIGHT_SPEAKER_GAIN,0,
				    AL_SYNC_INPUT_TO_AES,0,
				    AL_SYNC_OUTPUT_TO_AES,0
				  };
long		pvbuf[]		= { AL_INPUT_RATE,0,
				    AL_INPUT_SOURCE,0,
				    AL_LEFT_INPUT_ATTEN,0,
				    AL_RIGHT_INPUT_ATTEN,0,
				    AL_OUTPUT_RATE,0,
				    AL_LEFT_SPEAKER_GAIN,0,
				    AL_RIGHT_SPEAKER_GAIN,0,
				    AL_SYNC_INPUT_TO_AES,0,
				    AL_SYNC_OUTPUT_TO_AES,0
				  };
long		win_llx;
long		win_lly;
long		my_x;
long		my_y;
long		in_gain		= -1;
long		out_gain	= -1;
long		lgain_adjust	= 0;
long		gain_adjust	= 0;
char		autogain	= 0;
char		shhh		= 0;
char		listen		= 0;
long		rates[]		= { AL_RATE_8000, AL_RATE_11025, AL_RATE_16000,
				    AL_RATE_22050,AL_RATE_32000, AL_RATE_44100,
				    AL_RATE_48000 };

/* *****************************************************************************

	Routine:	main - This is the MAIN for the spectra program
	Author:		David Cook

***************************************************************************** */
void main( int argc, char **argv) 
{
long	i,j;
int	d1;
short	d2;

	screen_x_size = getgdesc(GD_XPMAX);
	screen_y_size = getgdesc(GD_YPMAX);

	llx = screen_x_size - BARWIDTH;
	lly = 0;

	/*  Open the window that contains the memstat display  */
	noborder();
	prefposition(llx,llx+BARWIDTH-1,0,screen_y_size-1);
	winopen("audio");

	ortho2(0.0,((float)BARWIDTH),0.0,((float)screen_y_size));
	viewport(0,BARWIDTH-1,0,screen_y_size-1);
	RGBmode();
	gconfig();
	drawmode(NORMALDRAW);
	cpack(0);
	clear();
	window_id = winget();
	getorigin(&win_llx,&win_lly);

	/*  Setup the values  */
	for (i=0; i<MAXDURATION; i++) {
	  for (j=0; j<ORDER; j++) {
	    amp[i][j] = 0.0;
	  }
	}

	for (i=0; i<8000; i++) ab[i] = (short) ((i*8)%256-128);

	if (ip_audio_open(CONFIGURE|AUDIO_IN|AUDIO_OUT) == -1) exit(-1);

	idur = 0;

	/*  Get the audio defaults and reset them to our desired values  */
	ALsetfillpoint(in_port,(ALgetqueuesize(audio_config)-CHUNK_SIZE));
	ALgetparams(AL_DEFAULT_DEVICE,pvbuf,18);
	if (pvbuf[11] != pvbuf[13]) {
	  if (pvbuf[11] > pvbuf[13]) pvbuf[13] = pvbuf[11];
	  else                       pvbuf[11] = pvbuf[13];
	  setparams();
	}
	if (pvbuf[5] != pvbuf[7]) {
	  if (pvbuf[5] > pvbuf[7]) pvbuf[5] = pvbuf[7];
	  else                     pvbuf[7] = pvbuf[5];
	  setparams();
	}

	/*  Draw the button and other stuff  */
	ip_font();
	draw_panel();

	/* ip_audio_reopen(); */

	qdevice(WINQUIT);
	qdevice(MOUSEX);
	qdevice(MOUSEY);
	qdevice(LEFTMOUSE);
	qdevice(MIDDLEMOUSE);
	qdevice(RIGHTMOUSE);

	while (1) {
	  if (qtest()) {
	    d1 = qread(&d2);
	    switch (d1) {
	      case WINQUIT:
	      case WINSHUT:
	        ip_audio_close(AUDIO_IN|AUDIO_OUT);
	        gexit();
	        exit(0);
	      break;

	      case MOUSEX:
	      case MOUSEY:
	        my_x = getvaluator(MOUSEX) - win_llx;
	        my_y = getvaluator(MOUSEY) - win_lly;
	      break;

	      case LEFTMOUSE:
	      case MIDDLEMOUSE:
	      case RIGHTMOUSE:
	        if (d2) parse_it();
	      break;

	      case INPUTCHANGE:
	        winset(d2);
	      break;

	      case REDRAW:
	        if (d2 == window_id) {
		  draw_panel();
	          getorigin(&win_llx,&win_lly);
	        }
	      break;

	      default:
	      break;
	    }
	  }

	  get_sample();

	  /*  See if any parameters have changes  */
	  ALgetparams(AL_DEFAULT_DEVICE,tvbuf,18);
	  j = 0;
	  for (i = 1; i < 18; i+=2) {
	    if (tvbuf[i] != pvbuf[i]) {
	      if (!(
	            (((i == 11) || (i == 13)) && (tvbuf[i] == 0) && shhh) ||
	            (((i == 5)  || (i == 7))  && 
	            ((tvbuf[i]==(pvbuf[i]-gain_adjust))||(tvbuf[i]==pvbuf[i])))
	         )){ 
	        pvbuf[i] = tvbuf[i];
	        j = 1;
	      }
	    }
	  }
	  if (pvbuf[11] != pvbuf[13]) {
	    if (pvbuf[11] > pvbuf[13]) pvbuf[13] = pvbuf[11];
	    else                       pvbuf[11] = pvbuf[13];
	    setparams();
	  }
	  if (pvbuf[5] != pvbuf[7]) {
	    if (pvbuf[5] > pvbuf[7]) pvbuf[5] = pvbuf[7];
	    else                     pvbuf[7] = pvbuf[5];
	    setparams();
	  }
	  if (j) {
	    draw_gains();
	    draw_icons();
	    draw_irate();
	    draw_orate();
	  }
	}
}

/* *****************************************************************************

	Routine:	ip_audio_open()
	Author:		David Cook

	Description:	This routine will open the audio ports for input and/or
			output.

	Syntax:		ip_audio_open(mode);

	Where:		mode = [long] A combination of OR'ED bit flags as:
					CONFIGURE = Used first time
					AUDIO_IN  = Creates an INPUT port
					AUDIO_OUT = Creates an OUTPUT port

	Returns:	 0 = No problem, ports opened properly
			-1 = Error occured opening the ports

***************************************************************************** */
long ip_audio_open(long mode)
{
	/*  See if this is a new occurance  */
	if (mode & CONFIGURE) {
	  audio_config = ALnewconfig();
	  ALsetwidth(audio_config,AL_SAMPLE_16);
	  ALsetchannels(audio_config,AL_MONO);
	  in_port = out_port = 0;
	}

	/*  Check for opening it for input  */
	if (mode & AUDIO_IN) {
	  if ((in_port = ALopenport("audio input","r",audio_config)) == 0) {
	    fprintf(stderr,"spectra: Couldn't open audio port for input!\n");
	    return(-1);
	  }
	}

	/*  Check for opening it for output  */
	if (mode & AUDIO_OUT) {
	  if ((out_port = ALopenport("audio output","w",audio_config)) == 0) {
	    fprintf(stderr,"spectra: Couldn't open audio port for output!\n");
	    return(-1);
	  }
	}
	return(0);
}

/* *****************************************************************************

	Routine:	ip_audio_close()
	Author:		David Cook

	Description:	This routine will close the currently open ports

	Syntax:		ip_audio_close(mode);

	Where:		mode = [long] A combination of OR'ED bit flags as:
					AUDIO_IN  = Closes an INPUT port
					AUDIO_OUT = Closes an OUTPUT port

***************************************************************************** */
void ip_audio_close(long mode)
{
	if ((mode & AUDIO_IN) && in_port) {
	  ALcloseport(in_port);
	  in_port = 0;
	}

	if ((mode & AUDIO_OUT) && out_port) {
	  ALcloseport(out_port);
	  out_port = 0;
	}
	return;
}

/* *****************************************************************************

	Routine:	ip_audio_reopen()
	Author:		David Cook

	Description:	This routine will close and reopen the current ports.

	Syntax:		ip_audio_reopen();

***************************************************************************** */
void ip_audio_reopen()
{
	if (in_port) {
	  ip_audio_close(AUDIO_IN);
	  ip_audio_open(AUDIO_IN);
	}
	if (out_port) {
	  ip_audio_close(AUDIO_OUT);
	  ip_audio_open(AUDIO_OUT);
	}
	return;
}

/*  This routine is taken from the amesh.c++ program (actually Power.c++)
    supplied by SGI  */
float spectrum(short *samples,int nsamples,float *ps,int nfreqs,float mp)
{
int	nfft;
int	i;
short	*pb;
float	pwr,nfftsqd;
complex	*pc;


	for (pb=samples, pc=tmpc, i=0; i<nsamples; i+=2) {
	  pc->r   = (float) *pb++;
	  pc->r  += (float) *pb++;
	  pc->i   = 0.0;
	  pc++;
	} 
	nsamples >>= 1;
	nfft       = 1;

	while (nfft < nsamples) nfft <<= 1;
	if (nfft > nsamples) {
	  for (i=0; i<(nfft-nsamples); i++) {
	    pc->r = 0.0;
	    pc->i = 0.0;
	    pc++;
	  }
	}
	pc = tmpc;
 
	NRICfft((float *)pc,nfft,1);
 	nfftsqd = ((float)nfft)*((float)nfft);

	for(i=1; i<nfreqs; i++) {
	  pwr  = tmpc[i].r * tmpc[i].r;
	  pwr += tmpc[i].i * tmpc[i].i;
	  pwr  = fsqrt(pwr/nfftsqd);
	  pwr *= 1.0 + (( (float)(i)/(float)nfreqs)*.3);
	  ps[i-1] = pwr;
	  if (pwr > mp) mp = pwr;
	}

	return(mp);
}

/* FFT from Numerical recipes in C page 411 */

void NRICfft(float *data,unsigned int nn,int isign) 
{
int		istep,m,mmax,i,j;
unsigned int	n;
double		wtemp,wr,wpr,wpi,wi,theta;
float		tempr,tempi;

	data -= 1;
	n     = nn << 1;
	j     = 1;

	for (i=1; i<n; i+=2) {
	  if (j>i) {
	    SWAP(data[j],data[i]);
	    SWAP(data[j+1],data[i+1]);
	  }
	  m = n >> 1;
	  while (m >= 2 && j > m) {
	    j -=m;
	    m >>= 1;
	  }
	  j +=m;
	}

	mmax = 2;
	while (n > mmax) {
	  istep = 2*mmax;
	  theta = PI2 / (isign*mmax);
	  wtemp = sin(0.5*theta);
	  wpr = -2.0*wtemp*wtemp;
	  wpi = sin(theta);
	  wr = 1.0;
	  wi = 0.0;
	  for (m=1; m<mmax; m+=2) {
	    for (i=m; i<n; i+=istep) {
	      j = i+mmax;
	      tempr = wr*data[j]-wi*data[j+1];
	      tempi = wr*data[j+1]+wi*data[j];
	      data[j] = data[i]-tempr;
	      data[j+1] = data[i+1]-tempi;
	      data[i] += tempr;
	      data[i+1] += tempi;
            }
	    wr = (wtemp=wr)*wpr-wi*wpi+wr;
	    wi = wi*wpr+wtemp*wpi+wi;
	  }
	  mmax = istep;
	}
	return;
}

/* *****************************************************************************

	Routine:	ge_draw_basic_button    << ge Services >>
	Programmer:	David Cook - cookware

	Copyright:	This routine belongs to the Interplay menu manager
			system.  It was removed and slightly modified to
			allow memstat to be distributed without out the
			overhead of Interplay.  Note that the look and feel
			of this routine are the copyright of cookware, and
			are part of the Interplay menu manager.  For more
			information about Interplay, contact David Cook at
			(317) 769-5049

	Description:	This routine will draw a basic button, with no text.

        Syntax:		ge_draw_basic_button(llx,lly,width,height);

	Where:		llx    - (int) Relative LL X of the button
			lly    - (int) Relative LL Y of the button
			width  - (int) Width of the button
			height - (int) Height of the button

***************************************************************************** */
void	ge_draw_basic_button(int x,int y,int w,int h)
{
long	a[6][2];

	concave(TRUE);

	/*  Create the button polygon data  */
	a[0][0] = a[1][0] = x;
	a[0][1] = a[4][1] = a[5][1] = y;
	a[1][1] = a[2][1] = a[3][1] = y + h;
	a[3][0] = a[4][0] = x + w;
	a[2][0] = a[5][0] = x + (w / 2);

	/*  Draw the button  */
	bgnpolygon();
	  cpack(0);
	  v2i(a[0]);
	  v2i(a[1]);
	  cpack(0x6f6f6f);
	  v2i(a[2]);
	  v2i(a[5]);
	  cpack(0);
	  v2i(a[0]);
	endpolygon();

	bgnpolygon();
	  cpack(0x6f6f6f);
	  v2i(a[5]);
	  cpack(0);
	  v2i(a[4]);
	  v2i(a[3]);
	  cpack(0x6f6f6f);
	  v2i(a[2]);
	  v2i(a[5]);
	endpolygon();

	return;
}

/* *****************************************************************************

	Routine:	update_amp()
	Programmer:	David Cook - cookware

	Description:	This routine will update the amplitude display on the
			slider.

	Syntax:		update_amp();

***************************************************************************** */
void update_amp()
{
long	a,y;

	if ((lpwr != pwr) || (lmaxpwr != maxpwr)) {
	  lpwr = pwr;
	  lmaxpwr = maxpwr;

	  /*  Show the MAX POWER setting  */
	  if (maxpwr < BIGEST) {
	    a = FALSE;
	    y = (long)(maxpwr / NUMBER);
	    cpack(0xffff00);
	    tmaxpwr = maxpwr;
	  } else {
	    a = TRUE;
	    y = (long)(tmaxpwr / NUMBER);
	    cpack(0x0000ff);
	  }
	  if (y < 0)   y = 0;
	  if (y > 255) y = 255;
	  rectfi(10,y+10+BS,12,y+10+BS);

	  /*  Show the black region above  */
	  if ((y+11+BS) <= (264+BS)) {
	    cpack(0);
	    rectfi(10,y+11+BS,12,264+BS);
	  }

	  /*  Show the area below this setting  */
	  cpack((a) ? 0x1f1f3f : 0x1f3f1f);
	  rectfi(10,10+BS,12,y+9+BS);

	  /*  Show the fast line  */
	  if (pwr < BIGEST) {
	    y = (long)(pwr / NUMBER);
	    tpwr = pwr;
	  } else {
	    y = (long)(tpwr / NUMBER);
	    set_autogain(TRUE);
	  }
	  if (y > 255) y = 255;
	  if (y < 0)   y = 0;
	  cpack (0x00afaf);
	  rectfi(11,10+BS,11,y+10+BS);
	}

	return;
}

/* *****************************************************************************

	Routine:	get_sample()
	Programmer:	David Cook - cookware

	Description:	This routine will get a sample from the audio system
			and will set the VU meter accordingly.

	Syntax:		get_sample();

***************************************************************************** */
void get_sample()
{
long	n1,n2;

	/*  Update the display  */
	idur++;
	if (idur >= MAXDURATION) idur = 0;

	n1 = ALgetfilled(in_port);
	n2 = (n1 > AUDIO_BUF_SIZE) ? AUDIO_BUF_SIZE : n1;

	ALreadsamps(in_port,(void *)ab,n2);
	if (listen) ALwritesamps(out_port,(void*)ab,n2);

	pwr = spectrum(ab+(n2-CHUNK_SIZE),CHUNK_SIZE,&amp[idur][0],ORDER,0);

	if (pwr < maxpwr) nframe++;
	if (nframe > DURATION) {
	  nframe = 0;
	  maxpwr *= 0.97;
	}
	if (pwr > maxpwr) {
	  maxpwr = pwr;
	  nframe = 0;
	}
	update_amp();

	return;
}

/* *****************************************************************************

	Routine:	ip_font()
	Programmer:	David Cook - cookware

	Copyright:	This routine belongs to the Interplay menu manager
			system.  It was removed and slightly modified to
			allow memstat to be distributed without out the
			overhead of Interplay.  Note that the look and feel
			of this routine are the copyright of cookware, and
			are part of the Interplay menu manager.  For more
			information about Interplay, contact David Cook at
			(317) 769-5049

	Description:	This routine handles attaching the raster font to the
			font system for use with the memstat program.

	Syntax:		ip_font();

***************************************************************************** */
void	ip_font()
{
	defrasterfont(2,16,256,chars,1099,raster_f);
	font(2);
	return;
}

/* *****************************************************************************

	Routine:	draw_panel()
	Programmer:	David Cook - cookware

	Description:	Draw the entire panel on the screen.

	Syntax:		draw_panel();

***************************************************************************** */
void draw_panel()
{
long	i;

	/*  Draw the basic outline  */
        ge_draw_basic_button(0,0,BARWIDTH,screen_y_size);
	cpack(0);
	rectfi(0,527,BARWIDTH-1,527);
	rectfi(0,489,BARWIDTH-1,489);
	rectfi(0,411,BARWIDTH-1,411);
	rectfi(0,373,BARWIDTH-1,373);
	rectfi(0,335,BARWIDTH-1,335);
	rectfi(9,9+BS,13,265+BS);
	update_amp();

	/*  Draw the copyright text  */
	i = screen_y_size - 125;
	cpack(0x252525);
	cmov2i(6,i+(10*0)+0);  charstr("\1");
	cmov2i(6,i+(10*1)+6);  charstr("\011");
	cmov2i(6,i+(10*2)+0);  charstr("\012");
	cmov2i(6,i+(10*3)+0);  charstr("\013");
	cmov2i(8,i+(10*4)+3);  charstr("\2");
	cmov2i(8,i+(10*5)+3);  charstr("\3");
	cmov2i(8,i+(10*6)+3);  charstr("\3");
	cmov2i(4,i+(10*7)+3);  charstr("\4");
	cmov2i(8,i+(10*8)+2);  charstr("\5");
	cmov2i(8,i+(10*9)+5);  charstr("\6");
	cmov2i(8,i+(10*10)+4); charstr("\7");
	cmov2i(8,i+(10*11)+0); charstr("\010");
	cpack(0x00000000);
	rectfi(0,i-6,BARWIDTH-1,i-6);

	/*  Draw the other goodies  */
	out_gain = in_gain = -1;
	draw_gains();
	draw_icons();
	draw_irate();
	draw_orate();

	return;
}

/* *****************************************************************************

	Routine:	draw_gains()
	Programmer:	David Cook - cookware

	Description:	Draw the gain entries

	Syntax:		draw_gains();

***************************************************************************** */
void draw_gains()
{
	/*  Draw the OUTPUT gain  */
	if (out_gain != pvbuf[11]) {
	  cpack(0);
	  rectfi(13,9+out_gain+BS,13,10+out_gain+BS);
	  out_gain = pvbuf[11];
	  cpack(0xff7fff);
	  rectfi(13,9+out_gain+BS,13,10+out_gain+BS);
	}

	/*  Draw the INPUT gain  */
	if ((in_gain != (255-pvbuf[5])) || (lgain_adjust != gain_adjust)) {
	  cpack(0);
	  if (autogain) rectfi(9,9+in_gain+BS,9,10+in_gain+BS);
	  rectfi(9,9+in_gain+BS-lgain_adjust,9,10+in_gain+BS-lgain_adjust);
	  in_gain = (255-pvbuf[5]);
	  lgain_adjust = gain_adjust;
	  if (autogain) {
	    cpack(0x00afaf);
	    rectfi(9,9+in_gain+BS,9,10+in_gain+BS);
	  }
	  cpack(0xff7fff);
	  rectfi(9,9+in_gain+BS-gain_adjust,9,10+in_gain+BS-gain_adjust);
	}
	return;
}

/* *****************************************************************************

	Routine:	draw_icons()
	Programmer:	David Cook - cookware

	Description:	Draw the various screen icons

	Syntax:		draw_icons();

***************************************************************************** */
void draw_icons()
{
	cpack((pvbuf[3] == AL_INPUT_MIC) ? HIGHLIGHT : LOWLIGHT);
	cmov2i(5,600); charstr("\014");

	cpack((pvbuf[3] == AL_INPUT_LINE) ? HIGHLIGHT : LOWLIGHT);
	cmov2i(5,570); charstr("\016");

	cpack((pvbuf[3] == AL_INPUT_DIGITAL) ? HIGHLIGHT : LOWLIGHT);
	cmov2i(5,540); charstr("\015");

	cpack((listen) ? 0x00005f : LOWLIGHT);
	cmov2i(5,500); charstr("\036");

	cpack((shhh) ? 0x00005f : LOWLIGHT);
	cmov2i(5,383); charstr("\017");

	cpack((autogain) ? 0x00005f : LOWLIGHT);
	cmov2i(6,342); charstr("\037");


	return;
}

/* *****************************************************************************

	Routine:	draw_irate()
	Programmer:	David Cook - cookware

	Description:	Draw the current input sample rate

	Syntax:		draw_irate();

***************************************************************************** */
void draw_irate()
{

	cpack(0x1f1f1f);
	cmov2i(8,467); charstr("\034");
	cpack(0x0);
	rectfi(4,453,BARWIDTH-4,463);
	cpack(0x00009f);
	if (pvbuf[3] == AL_INPUT_DIGITAL) {
	  cmov2i(4,455); charstr("\033");
	} else {
	  switch(pvbuf[1]) {
	    case AL_RATE_48000:
	      cmov2i(7,455); charstr("\024");
	    break;
  
	    case AL_RATE_44100:
	      cmov2i(7,455); charstr("\025");
	    break;

	    case AL_RATE_32000:
	      cmov2i(7,455); charstr("\026");
	    break;

	    case AL_RATE_22050:
	      cmov2i(7,455); charstr("\027");
	    break;

	    case AL_RATE_16000:
	      cmov2i(7,455); charstr("\030");
	    break;

	    case AL_RATE_11025:
	      cmov2i(7,455); charstr("\031");
	    break;

	    case AL_RATE_8000:
	      cmov2i(7,455); charstr("\032");
	    break;
	  }
	}
	return;
}

/* *****************************************************************************

	Routine:	draw_orate()
	Programmer:	David Cook - cookware

	Description:	Draw the current output sample rate

	Syntax:		draw_orate();

***************************************************************************** */
void draw_orate()
{

	cpack(0x1f1f1f);
	cmov2i(9,421); charstr("\035");
	cpack(0x0);
	rectfi(4,436,BARWIDTH-4,446);
	cpack(0x00009f);
	switch(pvbuf[9]) {
	  case AL_RATE_48000:
	    cmov2i(7,438); charstr("\024");
	  break;

	  case AL_RATE_44100:
	    cmov2i(7,438); charstr("\025");
	  break;

	  case AL_RATE_32000:
	    cmov2i(7,438); charstr("\026");
	  break;

	  case AL_RATE_22050:
	    cmov2i(7,438); charstr("\027");
	  break;

	  case AL_RATE_16000:
	    cmov2i(7,438); charstr("\030");
	  break;

	  case AL_RATE_11025:
	    cmov2i(7,438); charstr("\031");
	  break;

	  case AL_RATE_8000:
	    cmov2i(7,438); charstr("\032");
	  break;
	}
	return;
}

/* *****************************************************************************

	Routine:	wait_4_user()
	Programmer:	David Cook - cookware

	Description:	Draw the various screen icons

	Syntax:		wait_4_user();

***************************************************************************** */
void wait_4_user()
{
	while (BUTTONDOWN) get_sample();
	qreset();

	return;
}

/* *****************************************************************************

	Routine:	setparams()
	Programmer:	David Cook - cookware

	Description:	Call ALsetparams with various changes

	Syntax:		setparams();

***************************************************************************** */
void setparams()
{
long	i,j;

	if (!shhh) {
	  j = pvbuf[5];
	  pvbuf[5] = pvbuf[7] = j + gain_adjust;
	  if (pvbuf[5] < 0) pvbuf[5] = pvbuf[7] = 255;
	  ALsetparams(AL_DEFAULT_DEVICE,pvbuf,18);
	  pvbuf[5] = pvbuf[7] = j;
	} else {
	  i = pvbuf[11];
	  j = pvbuf[5];
	  pvbuf[11] = pvbuf[13] = 0;
	  pvbuf[5]  = pvbuf[7]  = j + gain_adjust;
	  if (pvbuf[5] < 0) pvbuf[5] = pvbuf[7] = 255;
	  ALsetparams(AL_DEFAULT_DEVICE,pvbuf,18);
	  pvbuf[5]  = pvbuf[7]  = j;
	  pvbuf[11] = pvbuf[13] = i;
	}
	return;
}

/* *****************************************************************************

	Routine:	set_autogain()
	Programmer:	David Cook - cookware

	Description:	This routine handles setting of the AUTOGAIN capability
			AUTOGAIN allows the program to automatically lessen
			the input gain (or output if this is digital) to avoid
			clipping.

	Syntax:		set_autogain(flag);

	Where:		flag = (long) FALSE - If current signal NOT clipped
				      TRUE  - If current signal IS  clipped

***************************************************************************** */
void set_autogain(long flag)
{
	if (autogain) {
	  if (flag) {
	    gain_adjust++;
	    setparams();
	    draw_gains();
	  }
	}
	return;
}

/* *****************************************************************************

	Routine:	parse_it()
	Programmer:	David Cook - cookware

	Description:	Determine if a 'strategic' area was hit.

	Syntax:		parse_it();

***************************************************************************** */
void parse_it()
{
long	i;

	/*  Determine where I am  */
	if ((my_y >= (7+BS)) && (my_y <= (267+BS))) {
	  if (my_x >= (BARWIDTH/2)) {
	    /*  This is the OUTPUT GAIN slider  */
	    while (BUTTONDOWN) {
	      my_x = getvaluator(MOUSEX) - win_llx;
	      my_y = getvaluator(MOUSEY) - win_lly;
	      i = my_y - 10 - BS;
	      if (i < 0)   i = 0;
	      if (i > 255) i = 255;
	      pvbuf[11] = pvbuf[13] = i;
	      setparams();
	      draw_gains();
	      get_sample();
	    }
	  } else {
	    /*  This is the INPUT GAIN slider  */
	    while (BUTTONDOWN) {
	      gain_adjust = 0;
	      my_x = getvaluator(MOUSEX) - win_llx;
	      my_y = getvaluator(MOUSEY) - win_lly;
	      i = my_y - 10 - BS;
	      if (i < 0)   i = 0;
	      if (i > 255) i = 255;
	      pvbuf[5] = pvbuf[7] = 255-i;
	      setparams();
	      draw_gains();
	      get_sample();
	    }
	  }
	} else if (my_y > (screen_y_size-131)) {
	  /*  User wants to quit!  */
	  ip_audio_close(AUDIO_IN|AUDIO_OUT);
	  gexit();
	  exit(0);
	} else if (my_y > 527) {
	  /*  User is switching the input source  */
	  if      ((my_y > 595) && (my_y < (595+25))) pvbuf[3]=AL_INPUT_MIC;
	  else if ((my_y > 565) && (my_y < (565+25))) pvbuf[3]=AL_INPUT_LINE;
	  else if ((my_y > 535) && (my_y < (535+25))) pvbuf[3]=AL_INPUT_DIGITAL;
	  setparams();
	  draw_icons();
	  draw_irate();
	  wait_4_user();
	} else if (my_y > 489) {
	  /*  Handle the LISTEN command  */
	  listen = !listen;
	  draw_icons();
	  wait_4_user();
	} else if (my_y > 453) {
	  /*  Handle the INPUT RATE  */
	  if (pvbuf[3] != AL_INPUT_DIGITAL) {
	    for (i = 0; i < 7; i++) if (rates[i] == pvbuf[1]) break;
	    if (i == 7) i = 5;
	    i++;
	    if (i == 7) i = 0;
	    pvbuf[1] = rates[i];
	    setparams();
	    draw_irate();
	  }
	  wait_4_user();
	} else if (my_y > 411) {
	  /*  Handle the OUTPUT RATE  */
	  for (i = 0; i < 7; i++) if (rates[i] == pvbuf[9]) break;
	  if (i == 7) i = 5;
	  i++;
	  if (i == 7) i = 0;
	  pvbuf[9] = rates[i];
	  setparams();
	  draw_orate();
	  wait_4_user();
	} else if (my_y > 373) {
	  /*  Handle the SHHH button  */
	  shhh = (!shhh);
	  setparams();
	  draw_icons();
	  wait_4_user();
	} else if (my_y > 335) {
	  /*  Handle the AUTOGAIN button  */
	  autogain = (!autogain);
	  lgain_adjust = gain_adjust = 0;
	  setparams();
	  draw_icons();
	  wait_4_user();
	}
	qreset();
	return;
}
unsigned short	raster_f[] =	{
					/*  ASCII 0d  */
					/*  ASCII 1d  */
					0x1f80,
					0x2040,
					0x4020,
					0x8f10,
					0x9090,
					0x9090,
					0x9090,
					0x9090,
					0x8010,
					0x4020,
					0x2040,
					0x1f80,
					/*  ASCII 2d  */
					0x3e00,
					0x7f00,
					0xe380,
					0xc180,
					0xc180,
					0xe380,
					0x6300,
					0x2200,
					/*  ASCII 3d  */
					0x3e00,
					0x7f00,
					0xe380,
					0xc180,
					0xc180,
					0xe380,
					0x7f00,
					0x3e00,
					/*  ASCII 4d  */
					0xfff0,
					0xfff0,
					0x0380,
					0x0700,
					0x0fc0,
					0x1df0,
					0x1870,
					0x1010,
					/*  ASCII 5d  */
					0xc000,
					0xf800,
					0x3e00,
					0x0780,
					0x0f80,
					0xfc00,
					0xfc00,
					0x0f80,
					0x0780,
					0x3e00,
					0xf800,
					0xc000,
					/*  ASCII 6d  */
					0x6700,
					0xef80,
					0xcd80,
					0xc980,
					0xd900,
					0xff80,
					0x7f80,
					0x0080,
					/*  ASCII 7d  */
					0xff80,
					0xff80,
					0x6000,
					0xc000,
					0xc000,
					/*  ASCII 8d  */
					0x3e00,
					0x7f00,
					0xdb80,
					0xd980,
					0xd980,
					0xdb80,
					0x7b00,
					0x3a00,
					/*  ASCII 9d  */
					0xd000,
					0xe000,
					/*  ASCII 10d  */
					0x3c40,
					0x7e60,
					0xe770,
					0xc330,
					0xc330,
					0xe670,
					0x7fe0,
					0x3fc0,
					/*  ASCII 11d  */
					0x3830,
					0x7870,
					0xe0f0,
					0xc1f0,
					0xc3b0,
					0xe730,
					0x7e30,
					0x3c30,
					/*  ASCII 12d  */
					0x0200,
					0x0200,
					0x0200,
					0x0200,
					0x3fc0,
					0x4020,
					0x8f10,
					0x9f90,
					0xbfd0,
					0xbfd0,
					0xbfd0,
					0xbfd0,
					0xbfd0,
					0x7fe0,
					0x2040,
					0x2040,
					0x2040,
					0x2040,
					0x2040,
					0x1080,
					0x0f00,
					/*  ASCII 13d  */
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x2020,
					0x4890,
					0x9048,
					0xa028,
					0xa228,
					0xa728,
					0xafa8,
					0xa728,
					0xa228,
					0xa028,
					0x9048,
					0x4890,
					0x2020,
					/*  ASCII 14d  */
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x2020,
					0x4890,
					0x9048,
					0xa028,
					0xa128,
					0xa328,
					0xa728,
					0xa328,
					0xa128,
					0xa028,
					0x9048,
					0x4890,
					0x2020,
					/*  ASCII 15d  */
					0x0258,
					0x05a8,
					0x0aa8,
					0x16a8,
					0x0c90,
					0x0448,
					0x0370,
					0x0140,
					0x0140,
					0x0140,
					0x0140,
					0x3f60,
					0x4150,
					0xff78,
					0x4290,
					0x3de0,
					/*  ASCII 16d  */
					0x0000,0x0000,
					0x0000,0x0000,
					0x0000,0x0000,
					0x0000,0x0000,
					0x0000,0x0000,
					0x0000,0x0000,
					0x0000,0x0000,
					0x0000,0x0000,
					0x0000,0x0000,
					0x0000,0x0000,
					0x0000,0x0000,
					0x0000,0x0000,
					0x0000,0x0000,
					0x00c0,0x0000,
					0x012f,0x1820,
					0x0021,0x2420,
					0x00e1,0x2420,
					0x012f,0x3820,
					0x0128,0x2020,
					0x00c8,0x2420,
					0x000f,0x19e0,
					0x0000,0x0000,
					0x0000,0x0000,
					0x08cf,0x318e,
					0x0928,0x4a44,
					0x0924,0x0a44,
					0x3cc2,0x3244,
					0x2921,0x0a44,
					0x2929,0x4a4c,
					0x20c6,0x3184,
					/*  ASCII 17d  */
					/*  ASCII 18d  */
					0x0000,0x0000,
					/*  ASCII 19d  */
					/*  ASCII 20d  */
					0x2300,
					0x2480,
					0x2480,
					0xf300,
					0xa480,
					0xa480,
					0x8300,
					/*  ASCII 21d  */
					0x2100,
					0x2100,
					0x2100,
					0xf780,
					0xa500,
					0xa500,
					0x8400,
					/*  ASCII 22d  */
					0x6780,
					0x9400,
					0x1200,
					0x6100,
					0x1080,
					0x9480,
					0x6300,
					/*  ASCII 23d  */
					0xf780,
					0x8400,
					0x4200,
					0x2100,
					0x1080,
					0x9480,
					0x6300,
					/*  ASCII 24d  */
					0x7300,
					0x2480,
					0x2480,
					0x2700,
					0x2400,
					0x6480,
					0x2300,
					/*  ASCII 25d  */
					0x7700,
					0x2200,
					0x2200,
					0x2200,
					0x2200,
					0x6600,
					0x2200,
					/*  ASCII 26d  */
					0x1800,
					0x2400,
					0x2400,
					0x1800,
					0x2400,
					0x2400,
					0x1800,
					/*  ASCII 27d  */
					0x9798,
					0x9424,
					0x9404,
					0xf718,
					0x9420,
					0x9424,
					0x6798,
					/*  ASCII 28d  */
					0x0400,
					0x0c00,
					0x1c00,
					0x3c00,
					0x7c00,
					0xfc00,
					0x7c00,
					0x3c00,
					0x1c00,
					0x0c00,
					0x0400,
					/*  ASCII 29d  */
					0x8000,
					0xc000,
					0xe000,
					0xf000,
					0xf800,
					0xfc00,
					0xf800,
					0xf000,
					0xe000,
					0xc000,
					0x8000,
					/*  ASCII 30d  */
					0x3e00,
					0x4100,
					0x8100,
					0x8880,
					0x5440,
					0x5220,
					0x5120,
					0x4920,
					0x2920,
					0x2490,
					0x4890,
					0x4890,
					0x5190,
					0x9690,
					0x9090,
					0x8f20,
					0x4020,
					0x3040,
					0x0f80,
					/*  ASCII 31d  */
					0x60c0,
					0x9120,
					0x9120,
					0x9120,
					0x9120,
					0x9120,
					0x71c0,
					0x1b00,
					0x0a00,
					0x0e00,
					0x0e00,
					0x0a00,
					0x0e00,
					0x0a00,
					0x1b00,
					0x1b00,
					0x1b00,
					0x1b00,
					0x3180,
					0x3180,
					0x3180,
					0x60c0,
					0x60c0,
					0x60c0,
					0x40c0,
					/*  ASCII 32d  */
					0x0000,
					/*  ASCII 33d  */
					0xc000,
					0xc000,
					0x0000,
					0x8000,
					0x8000,
					0x8000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					/*  ASCII 34d  */
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x4800,
					0xd800,
					0xd800,
					0xd800,
					/*  ASCII 35d  */
					0x6c00,
					0x6c00,
					0x6c00,
					0xff00,
					0xff00,
					0x3600,
					0x3600,
					0x7f80,
					0x7f80,
					0x1b00,
					0x1b00,
					0x1b00,
					/*  ASCII 36d  */
					0x1000,
					0x7c00,
					0xfe00,
					0xd600,
					0x1600,
					0x1e00,
					0x3c00,
					0x7800,
					0xf000,
					0xd000,
					0xd600,
					0xfe00,
					0x7c00,
					0x1000,
					/*  ASCII 37d  */
					0x1060,
					0x08f0,
					0x0998,
					0x0598,
					0x04f0,
					0x0260,
					0x3200,
					0x7900,
					0xcd00,
					0xcc80,
					0x7880,
					0x3040,
					/*  ASCII 38d  */
					0x39c0,
					0x7f80,
					0xc700,
					0xc700,
					0xcf80,
					0x7dc0,
					0x38c0,
					0x3c00,
					0x6600,
					0x6600,
					0x7e00,
					0x3c00,
					/*  ASCII 39d  */
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x8000,
					0x4000,
					0xc000,
					0xc000,
					/*  ASCII 40d  */
					0x3000,
					0x7000,
					0x6000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0x6000,
					0x7000,
					0x3000,
					/*  ASCII 41d  */
					0xc000,
					0xe000,
					0x6000,
					0x3000,
					0x3000,
					0x3000,
					0x3000,
					0x3000,
					0x3000,
					0x3000,
					0x3000,
					0x3000,
					0x6000,
					0xe000,
					0xc000,
					/*  ASCII 42d  */
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x8800,
					0x7000,
					0x7000,
					0xa800,
					0x2000,
					/*  ASCII 43d  */
					0x0000,
					0x0000,
					0x0000,
					0x1800,
					0x1800,
					0x1800,
					0xff00,
					0xff00,
					0x1800,
					0x1800,
					0x1800,
					/*  ASCII 44d  */
					0x8000,
					0x4000,
					0x4000,
					0xc000,
					0xc000,
					/*  ASCII 45d  */
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0xf000,
					0xf000,
					/*  ASCII 46d  */
					0xc000,
					0xc000,
					/*  ASCII 47d  */
					0x8000,
					0x8000,
					0x8000,
					0xc000,
					0x4000,
					0x4000,
					0x6000,
					0x2000,
					0x2000,
					0x3000,
					0x1000,
					0x1000,
					/*  ASCII 48d  */
					0x3c00,
					0x7e00,
					0xe700,
					0xc300,
					0xc300,
					0xc300,
					0xc300,
					0xc300,
					0xc300,
					0xe700,
					0x7e00,
					0x3c00,
					/*  ASCII 49d  */
					0x1800,
					0x1800,
					0x1800,
					0x1800,
					0x1800,
					0x1800,
					0x1800,
					0x1800,
					0xf800,
					0xf800,
					0x1800,
					0x0800,
					/*  ASCII 50d  */
					0xff00,
					0xff00,
					0x7000,
					0x3800,
					0x1c00,
					0x0e00,
					0x0700,
					0xc300,
					0xc300,
					0xe700,
					0x7e00,
					0x3c00,
					/*  ASCII 51d  */
					0x3c00,
					0x7e00,
					0xe700,
					0xc300,
					0x0700,
					0x1e00,
					0x1e00,
					0x0700,
					0xc300,
					0xe700,
					0x7e00,
					0x3c00,
					/*  ASCII 52d  */
					0x0600,
					0x0600,
					0x0600,
					0xff00,
					0xff00,
					0xc600,
					0x6600,
					0x6600,
					0x3600,
					0x3600,
					0x1e00,
					0x0e00,
					/*  ASCII 53d  */
					0x3c00,
					0x7e00,
					0xe700,
					0x0300,
					0x0300,
					0x4700,
					0x7e00,
					0x7c00,
					0x3000,
					0x3000,
					0x3f00,
					0x3f00,
					/*  ASCII 54d  */
					0x3c00,
					0x7e00,
					0xe700,
					0xc300,
					0xc300,
					0xe700,
					0xfe00,
					0xdc00,
					0xc000,
					0xe700,
					0x7e00,
					0x3c00,
					/*  ASCII 55d  */
					0x3000,
					0x3000,
					0x3000,
					0x1800,
					0x1800,
					0x1800,
					0x0c00,
					0x0c00,
					0x0600,
					0x0600,
					0xff00,
					0xff00,
					/*  ASCII 56d  */
					0x3c00,
					0x7e00,
					0xe700,
					0xc300,
					0xe700,
					0x7e00,
					0x3c00,
					0x6600,
					0xc300,
					0xe700,
					0x7e00,
					0x3c00,
					/*  ASCII 57d  */
					0x3c00,
					0x7e00,
					0xe700,
					0x0300,
					0x3b00,
					0x7f00,
					0xe700,
					0xc300,
					0xc300,
					0xe700,
					0x7e00,
					0x3c00,
					/*  ASCII 58d  */
					0x0000,
					0x0000,
					0xc000,
					0xc000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0xc000,
					0xc000,
					/*  ASCII 59d  */
					0x8000,
					0x4000,
					0x4000,
					0xc000,
					0xc000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0xc000,
					0xc000,
					/*  ASCII 60d  */
					0x0000,
					0x0000,
					0x0300,
					0x0f00,
					0x3c00,
					0xe000,
					0xe000,
					0x3c00,
					0x0f00,
					0x0300,
					/*  ASCII 61d  */
					0x0000,
					0x0000,
					0x0000,
					0xff00,
					0xff00,
					0x0000,
					0x0000,
					0xff00,
					0xff00,
					/*  ASCII 62d  */
					0x0000,
					0x0000,
					0xc000,
					0xf000,
					0x3c00,
					0x0700,
					0x0700,
					0x3c00,
					0xf000,
					0xc000,
					/*  ASCII 63d  */
					0x1800,
					0x1800,
					0x0000,
					0x1800,
					0x1800,
					0x0c00,
					0x0e00,
					0x0600,
					0xc600,
					0xc600,
					0xfe00,
					0x7c00,
					/*  ASCII 64d  */
					0x0f80,
					0x38c0,
					0x6000,
					0x4660,
					0xc990,
					0x9998,
					0x9888,
					0x98c8,
					0x88c8,
					0xcc48,
					0x47d8,
					0x6010,
					0x3860,
					0x0fc0,
					/*  ASCII 65d  */
					0xc060,
					0xc060,
					0xffe0,
					0x7fc0,
					0x60c0,
					0x71c0,
					0x3180,
					0x3b80,
					0x1b00,
					0x1f00,
					0x0e00,
					0x0e00,
					/*  ASCII 66d  */
					0xfe00,
					0xff00,
					0xc380,
					0xc180,
					0xc380,
					0xff00,
					0xff00,
					0xc380,
					0xc180,
					0xc380,
					0xff00,
					0xfe00,
					/*  ASCII 67d  */
					0x1f00,
					0x3f80,
					0x71c0,
					0x60c0,
					0xe000,
					0xc000,
					0xc000,
					0xe000,
					0x60c0,
					0x71c0,
					0x3f80,
					0x1f00,
					/*  ASCII 68d  */
					0xfc00,
					0xff00,
					0xc380,
					0xc180,
					0xc1c0,
					0xc0c0,
					0xc0c0,
					0xc1c0,
					0xc180,
					0xc380,
					0xff00,
					0xfc00,
					/*  ASCII 69d  */
					0xff00,
					0xff00,
					0xc000,
					0xc000,
					0xc000,
					0xfe00,
					0xfe00,
					0xc000,
					0xc000,
					0xc000,
					0xff00,
					0xff00,
					/*  ASCII 70d  */
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xfe00,
					0xfe00,
					0xc000,
					0xc000,
					0xc000,
					0xff00,
					0xff00,
					/*  ASCII 71d  */
					0x1ec0,
					0x3fc0,
					0x71c0,
					0x60c0,
					0xe3c0,
					0xc3c0,
					0xc000,
					0xe000,
					0x60c0,
					0x71c0,
					0x3f80,
					0x1f00,
					/*  ASCII 72d  */
					0xc0c0,
					0xc0c0,
					0xc0c0,
					0xc0c0,
					0xc0c0,
					0xc0c0,
					0xffc0,
					0xffc0,
					0xc0c0,
					0xc0c0,
					0xc0c0,
					0xc0c0,
					/*  ASCII 73d  */
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					/*  ASCII 74d  */
					0x7c00,
					0xfe00,
					0xc600,
					0xc600,
					0x0600,
					0x0600,
					0x0600,
					0x0600,
					0x0600,
					0x0600,
					0x0600,
					0x0600,
					/*  ASCII 75d  */
					0xc0e0,
					0xc1c0,
					0xc380,
					0xc700,
					0xce00,
					0xfc00,
					0xf800,
					0xdc00,
					0xce00,
					0xc700,
					0xc380,
					0xc1c0,
					/*  ASCII 76d  */
					0xff00,
					0xff00,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					/*  ASCII 77d  */
					0xc460,
					0xce60,
					0xce60,
					0xca60,
					0xdb60,
					0xdb60,
					0xd160,
					0xf1e0,
					0xf1e0,
					0xe0e0,
					0xe0e0,
					0xe0e0,
					/*  ASCII 78d  */
					0xc1c0,
					0xc3c0,
					0xc3c0,
					0xc6c0,
					0xc6c0,
					0xccc0,
					0xccc0,
					0xd8c0,
					0xd8c0,
					0xf0c0,
					0xf0c0,
					0xe0c0,
					/*  ASCII 79d  */
					0x1f00,
					0x3f80,
					0x71c0,
					0x60c0,
					0xe0e0,
					0xc060,
					0xc060,
					0xe0e0,
					0x60c0,
					0x71c0,
					0x3f80,
					0x1f00,
					/*  ASCII 80d  */
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xfe00,
					0xff00,
					0xc380,
					0xc180,
					0xc180,
					0xc380,
					0xff00,
					0xfe00,
					/*  ASCII 81d  */
					0x0080,
					0x1fc0,
					0x3f80,
					0x73c0,
					0x67c0,
					0xe260,
					0xc060,
					0xc060,
					0xe0e0,
					0x60c0,
					0x71c0,
					0x3f80,
					0x1f00,
					/*  ASCII 82d  */
					0xc1c0,
					0xc1c0,
					0xc180,
					0xc180,
					0xc380,
					0xff00,
					0xff00,
					0xc380,
					0xc180,
					0xc180,
					0xff80,
					0xff00,
					/*  ASCII 83d  */
					0x3e00,
					0x7f00,
					0xe380,
					0xc180,
					0x0380,
					0x1f00,
					0x7e00,
					0xf000,
					0xc180,
					0xe380,
					0x7f00,
					0x3e00,
					/*  ASCII 84d  */
					0x0c00,
					0x0c00,
					0x0c00,
					0x0c00,
					0x0c00,
					0x0c00,
					0x0c00,
					0x0c00,
					0x0c00,
					0x0c00,
					0xffc0,
					0xffc0,
					/*  ASCII 85d  */
					0x3f00,
					0x7f80,
					0xe1c0,
					0xc0c0,
					0xc0c0,
					0xc0c0,
					0xc0c0,
					0xc0c0,
					0xc0c0,
					0xc0c0,
					0xc0c0,
					0xc0c0,
					/*  ASCII 86d  */
					0x0c00,
					0x0c00,
					0x1e00,
					0x1e00,
					0x3300,
					0x3300,
					0x3300,
					0x6180,
					0x6180,
					0x6180,
					0xc0c0,
					0xc0c0,
					/*  ASCII 87d  */
					0x1860,
					0x1860,
					0x1860,
					0x3cf0,
					0x3cf0,
					0x34b0,
					0x6798,
					0x6798,
					0x6318,
					0xc30c,
					0xc30c,
					0xc30c,
					/*  ASCII 88d  */
					0xc180,
					0xe380,
					0x6300,
					0x3600,
					0x3e00,
					0x1c00,
					0x1c00,
					0x3e00,
					0x3600,
					0x6300,
					0xe380,
					0xc180,
					/*  ASCII 89d  */
					0x0c00,
					0x0c00,
					0x0c00,
					0x0c00,
					0x0c00,
					0x1e00,
					0x1e00,
					0x3300,
					0x3300,
					0x6180,
					0xe1c0,
					0xc0c0,
					/*  ASCII 90d  */
					0xff00,
					0xff00,
					0xe000,
					0x6000,
					0x3000,
					0x3800,
					0x1c00,
					0x0c00,
					0x0600,
					0x0700,
					0xff00,
					0xff00,
					/*  ASCII 91d  */
					0xf000,
					0xf000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xf000,
					0xf000,
					/*  ASCII 92d  */
					0x1000,
					0x1000,
					0x1000,
					0x3000,
					0x2000,
					0x2000,
					0x6000,
					0x4000,
					0x4000,
					0xc000,
					0x8000,
					0x8000,
					/*  ASCII 93d  */
					0xf000,
					0xf000,
					0x3000,
					0x3000,
					0x3000,
					0x3000,
					0x3000,
					0x3000,
					0x3000,
					0x3000,
					0x3000,
					0x3000,
					0x3000,
					0xf000,
					0xf000,
					/*  ASCII 94d  */
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0xc300,
					0xc300,
					0x6600,
					0x6600,
					0x3c00,
					0x1800,
					0x1800,
					/*  ASCII 95d  */
					0xff80,
					/*  ASCII 96d  */
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0xc000,
					0xc000,
					0x8000,
					0x4000,
					/*  ASCII 97d  */
					0x7700,
					0xfe00,
					0xc600,
					0xe600,
					0x7e00,
					0x0e00,
					0xc600,
					0xfe00,
					0x7c00,
					/*  ASCII 98d  */
					0xdc00,
					0xfe00,
					0xe700,
					0xc300,
					0xc300,
					0xc300,
					0xe700,
					0xfe00,
					0xdc00,
					0xc000,
					0xc000,
					0xc000,
					/*  ASCII 99d  */
					0x3c00,
					0x7e00,
					0xe700,
					0xc000,
					0xc000,
					0xc000,
					0xe700,
					0x7e00,
					0x3c00,
					/*  ASCII 100d  */
					0x3b00,
					0x7f00,
					0xe700,
					0xc300,
					0xc300,
					0xc300,
					0xe700,
					0x7f00,
					0x3f00,
					0x0300,
					0x0300,
					0x0300,
					/*  ASCII 101d  */
					0x3c00,
					0x7e00,
					0xe700,
					0xc000,
					0xff00,
					0xff00,
					0xc300,
					0x7e00,
					0x3c00,
					/*  ASCII 102d  */
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0xf000,
					0xf000,
					0x6000,
					0x7000,
					0x3000,
					/*  ASCII 103d  */
					0x3c00,
					0x7e00,
					0xe700,
					0x0300,
					0x3b00,
					0x7f00,
					0xe700,
					0xc300,
					0xc300,
					0xc300,
					0xe700,
					0x7f00,
					0x3b00,
					/*  ASCII 104d  */
					0xc300,
					0xc300,
					0xc300,
					0xc300,
					0xc300,
					0xc300,
					0xe300,
					0xff00,
					0xde00,
					0xc000,
					0xc000,
					0xc000,
					/*  ASCII 105d  */
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0x0000,
					0xc000,
					0xc000,
					/*  ASCII 106d  */
					0xc000,
					0xe000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x0000,
					0x6000,
					0x6000,
					/*  ASCII 107d  */
					0xc700,
					0xc600,
					0xce00,
					0xec00,
					0xfc00,
					0xf800,
					0xdc00,
					0xce00,
					0xc700,
					0xc000,
					0xc000,
					0xc000,
					/*  ASCII 108d  */
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					/*  ASCII 109d  */
					0xc630,
					0xc630,
					0xc630,
					0xc630,
					0xc630,
					0xc630,
					0xe730,
					0xfff0,
					0xdee0,
					/*  ASCII 110d  */
					0xc300,
					0xc300,
					0xc300,
					0xc300,
					0xc300,
					0xc300,
					0xe300,
					0xff00,
					0xde00,
					/*  ASCII 111d  */
					0x3c00,
					0x7e00,
					0xe700,
					0xc300,
					0xc300,
					0xc300,
					0xe700,
					0x7e00,
					0x3c00,
					/*  ASCII 112d  */
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xdc00,
					0xfe00,
					0xe700,
					0xc300,
					0xc300,
					0xc300,
					0xe700,
					0xfe00,
					0xdc00,
					/*  ASCII 113d  */
					0x0300,
					0x0300,
					0x0300,
					0x0300,
					0x3b00,
					0x7f00,
					0xe700,
					0xc300,
					0xc300,
					0xc300,
					0xe700,
					0x7f00,
					0x3b00,
					/*  ASCII 114d  */
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xc000,
					0xe000,
					0xf800,
					0xd800,
					/*  ASCII 115d  */
					0x7c00,
					0xfe00,
					0xc600,
					0x0e00,
					0x7c00,
					0xf000,
					0xc600,
					0xfe00,
					0x7c00,
					/*  ASCII 116d  */
					0x3000,
					0x7000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0xf000,
					0xf000,
					0x6000,
					0x6000,
					/*  ASCII 117d  */
					0x7b00,
					0xff00,
					0xc700,
					0xc300,
					0xc300,
					0xc300,
					0xc300,
					0xc300,
					0xc300,
					/*  ASCII 118d  */
					0x1800,
					0x1800,
					0x3c00,
					0x3c00,
					0x6600,
					0x6600,
					0x6600,
					0xc300,
					0xc300,
					/*  ASCII 119d  */
					0x1980,
					0x1980,
					0x39c0,
					0x3fc0,
					0x6f60,
					0x6660,
					0x6660,
					0xc630,
					0xc630,
					/*  ASCII 120d  */
					0xc600,
					0xc600,
					0x6c00,
					0x7c00,
					0x3800,
					0x7c00,
					0x6c00,
					0xc600,
					0xc600,
					/*  ASCII 121d  */
					0x6000,
					0x7000,
					0x1800,
					0x1800,
					0x1800,
					0x1c00,
					0x3c00,
					0x3600,
					0x3600,
					0x6600,
					0x6300,
					0xc300,
					0xc300,
					/*  ASCII 122d  */
					0xfe00,
					0xfe00,
					0x7000,
					0x3800,
					0x1800,
					0x1c00,
					0x0e00,
					0xfe00,
					0xfe00,
					/*  ASCII 123d  */
					0x3000,
					0x7000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0xc000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x7000,
					0x3000,
					/*  ASCII 124d  */
					/*  ASCII 125d  */
					0xc000,
					0xe000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x3000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0x6000,
					0xe000,
					0xc000,
					/*  ASCII 126d  */
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x0000,
					0x8e00,
					0x9900,
					0x7100,
					/*  ASCII 127d  */
					/*  ASCII 128d  */
					/*  ASCII 129d  */
					/*  ASCII 130d  */
					/*  ASCII 131d  */
					/*  ASCII 132d  */
					/*  ASCII 133d  */
					/*  ASCII 134d  */
					/*  ASCII 135d  */
					/*  ASCII 136d  */
					/*  ASCII 137d  */
					/*  ASCII 138d  */
					/*  ASCII 139d  */
					/*  ASCII 140d  */
					/*  ASCII 141d  */
					/*  ASCII 142d  */
					/*  ASCII 143d  */
					/*  ASCII 144d  */
					/*  ASCII 145d  */
					/*  ASCII 146d  */
					/*  ASCII 147d  */
					/*  ASCII 148d  */
					/*  ASCII 149d  */
					/*  ASCII 150d  */
					/*  ASCII 151d  */
					/*  ASCII 152d  */
					/*  ASCII 153d  */
					/*  ASCII 154d  */
					/*  ASCII 155d  */
					/*  ASCII 156d  */
					/*  ASCII 157d  */
					/*  ASCII 158d  */
					/*  ASCII 159d  */
					/*  ASCII 160d  */
					/*  ASCII 161d  */
					/*  ASCII 162d  */
					/*  ASCII 163d  */
					/*  ASCII 164d  */
					/*  ASCII 165d  */
					/*  ASCII 166d  */
					/*  ASCII 167d  */
					/*  ASCII 168d  */
					/*  ASCII 169d  */
					/*  ASCII 170d  */
					/*  ASCII 171d  */
					/*  ASCII 172d  */
					/*  ASCII 173d  */
					/*  ASCII 174d  */
					/*  ASCII 175d  */
					/*  ASCII 176d  */
					/*  ASCII 177d  */
					/*  ASCII 178d  */
					/*  ASCII 179d  */
					/*  ASCII 180d  */
					/*  ASCII 181d  */
					/*  ASCII 182d  */
					/*  ASCII 183d  */
					/*  ASCII 184d  */
					/*  ASCII 185d  */
					/*  ASCII 186d  */
					/*  ASCII 187d  */
					/*  ASCII 188d  */
					/*  ASCII 189d  */
					/*  ASCII 190d  */
					/*  ASCII 191d  */
					/*  ASCII 192d  */
					/*  ASCII 193d  */
					/*  ASCII 194d  */
					/*  ASCII 195d  */
					/*  ASCII 196d  */
					/*  ASCII 197d  */
					/*  ASCII 198d  */
					/*  ASCII 199d  */
					/*  ASCII 200d  */
					/*  ASCII 201d  */
					/*  ASCII 202d  */
					/*  ASCII 203d  */
					/*  ASCII 204d  */
					/*  ASCII 205d  */
					/*  ASCII 206d  */
					/*  ASCII 207d  */
					/*  ASCII 208d  */
					/*  ASCII 209d  */
					/*  ASCII 210d  */
					/*  ASCII 211d  */
					/*  ASCII 212d  */
					/*  ASCII 213d  */
					/*  ASCII 214d  */
					/*  ASCII 215d  */
					/*  ASCII 216d  */
					/*  ASCII 217d  */
					/*  ASCII 218d  */
					/*  ASCII 219d  */
					/*  ASCII 220d  */
					/*  ASCII 221d  */
					/*  ASCII 222d  */
					/*  ASCII 223d  */
					/*  ASCII 224d  */
					/*  ASCII 225d  */
					/*  ASCII 226d  */
					/*  ASCII 227d  */
					/*  ASCII 228d  */
					/*  ASCII 229d  */
					/*  ASCII 230d  */
					/*  ASCII 231d  */
					/*  ASCII 232d  */
					/*  ASCII 233d  */
					/*  ASCII 234d  */
					/*  ASCII 235d  */
					0x0000,0x0000,0x0000,
					/*  ASCII 236d  */
					0x0000,0x0000,0x0000,
					/*  ASCII 237d  */
					/*  ASCII 238d  */
					0x0000,0x0000,0x0000,
					/*  ASCII 239d  */
					0x0000,0x0000,0x0000,
					/*  ASCII 240d  */
					/*  ASCII 241d  */
					0x0000,0x0000,0x0000,
					/*  ASCII 242d  */
					0x0000,0x0000,0x0000,
					/*  ASCII 243d  */
					0x0000,0x0000,0x0000,
					/*  ASCII 244d  */
					/*  ASCII 245d  */
					/*  ASCII 246d  */
					/*  ASCII 247d  */
					/*  ASCII 248d  */
					/*  ASCII 249d  */
					/*  ASCII 250d  */
					/*  ASCII 251d  */
					/*  ASCII 252d  */
					/*  ASCII 253d  */
					/*  ASCII 254d  */
					/*  ASCII 255d  */
				0 };

Fontchar	chars[256] =	{
					{ 0, 0, 0, 0, 0, 0 },
					{ 0, 12, 12, 0, 0, 13 },
					{ 12, 9, 8, 0, 0, 10 },
					{ 20, 9, 8, 0, 0, 10 },
					{ 28, 12, 8, 0, 0, 10 },
					{ 36, 9, 12, 0, 0, 14 },
					{ 48, 9, 8, 0, 0, 10 },
					{ 56, 9, 5, 0, 0, 7 },
					{ 61, 9, 8, 0, 0, 10 },
					{ 69, 4, 2, 0, 0, 4 },
					{ 71, 12, 8, 0, 0, 10 },
					{ 79, 12, 8, 0, 0, 10 },
					{ 87, 12, 21, 0, 0, 13 },
					{ 108, 13, 17, 0, 0, 13 },
					{ 125, 13, 17, 0, 0, 13 },
					{ 142, 13, 16, 0, 0, 13 },
					{ 158, 31, 30, 0, 0, 0 },
					{ 218, 0, 0, 0, 0, 0 },
					{ 218, 27, 1, 0, 0, 26 },
					{ 220, 0, 0, 0, 0, 0 },
					{ 220, 9, 7, 0, 0, 0 },
					{ 227, 9, 7, 0, 0, 0 },
					{ 234, 9, 7, 0, 0, 0 },
					{ 241, 9, 7, 0, 0, 0 },
					{ 248, 9, 7, 0, 0, 0 },
					{ 255, 8, 7, 0, 0, 0 },
					{ 262, 6, 7, 0, 0, 0 },
					{ 269, 14, 7, 0, 0, 0 },
					{ 276, 6, 11, 0, 0, 0 },
					{ 287, 6, 11, 0, 0, 0 },
					{ 298, 12, 19, 0, 0, 0 },
					{ 317, 11, 25, 0, 0, 0 },
					{ 342, 8, 1, 0, 0, 7 },
					{ 343, 2, 12, 0, 0, 4 },
					{ 355, 5, 14, 0, 0, 7 },
					{ 369, 9, 12, 0, 0, 11 },
					{ 381, 7, 14, 0, -1, 9 },
					{ 395, 13, 12, 0, 0, 15 },
					{ 407, 10, 12, 0, 0, 12 },
					{ 419, 2, 14, 0, 0, 4 },
					{ 433, 4, 15, 0, -1, 6 },
					{ 448, 4, 15, 0, -1, 6 },
					{ 463, 5, 10, 0, 0, 7 },
					{ 473, 8, 11, 0, 0, 10 },
					{ 484, 2, 5, 0, -3, 4 },
					{ 489, 4, 6, 0, 0, 6 },
					{ 495, 2, 2, 0, 0, 4 },
					{ 497, 4, 12, 0, 0, 6 },
					{ 509, 8, 12, 0, 0, 10 },
					{ 521, 5, 12, 0, 0, 7 },
					{ 533, 8, 12, 0, 0, 10 },
					{ 545, 8, 12, 0, 0, 10 },
					{ 557, 8, 12, 0, 0, 10 },
					{ 569, 8, 12, 0, 0, 10 },
					{ 581, 8, 12, 0, 0, 10 },
					{ 593, 8, 12, 0, 0, 10 },
					{ 605, 8, 12, 0, 0, 10 },
					{ 617, 8, 12, 0, 0, 10 },
					{ 629, 2, 10, 0, 0, 4 },
					{ 639, 2, 11, 0, -3, 4 },
					{ 650, 8, 10, 0, 0, 10 },
					{ 660, 8, 9, 0, 0, 10 },
					{ 669, 8, 10, 0, 0, 10 },
					{ 679, 7, 12, 0, 0, 9 },
					{ 691, 13, 14, 0, 0, 15 },
					{ 705, 11, 12, 0, 0, 13 },
					{ 717, 9, 12, 0, 0, 11 },
					{ 729, 10, 12, 0, 0, 12 },
					{ 741, 10, 12, 0, 0, 12 },
					{ 753, 8, 12, 0, 0, 10 },
					{ 765, 8, 12, 0, 0, 10 },
					{ 777, 10, 12, 0, 0, 12 },
					{ 789, 10, 12, 0, 0, 12 },
					{ 801, 2, 12, 0, 0, 4 },
					{ 813, 7, 12, 0, 0, 9 },
					{ 825, 11, 12, 0, 0, 13 },
					{ 837, 8, 12, 0, 0, 10 },
					{ 849, 11, 12, 0, 0, 13 },
					{ 861, 10, 12, 0, 0, 12 },
					{ 873, 11, 12, 0, 0, 13 },
					{ 885, 9, 12, 0, 0, 11 },
					{ 897, 11, 13, 0, -1, 13 },
					{ 910, 10, 12, 0, 0, 12 },
					{ 922, 9, 12, 0, 0, 11 },
					{ 934, 10, 12, 0, 0, 12 },
					{ 946, 10, 12, 0, 0, 12 },
					{ 958, 10, 12, 0, 0, 12 },
					{ 970, 14, 12, 0, 0, 16 },
					{ 982, 9, 12, 0, 0, 11 },
					{ 994, 10, 12, 0, 0, 12 },
					{ 1006, 8, 12, 0, 0, 10 },
					{ 1018, 4, 15, 0, 0, 6 },
					{ 1033, 4, 12, 0, 0, 6 },
					{ 1045, 4, 15, 0, 0, 6 },
					{ 1060, 8, 16, 0, 0, 10 },
					{ 1076, 9, 1, 0, -2, 11 },
					{ 1077, 2, 14, 0, 0, 4 },
					{ 1091, 8, 9, 0, 0, 10 },
					{ 1100, 8, 12, 0, 0, 10 },
					{ 1112, 8, 9, 0, 0, 10 },
					{ 1121, 8, 12, 0, 0, 10 },
					{ 1133, 8, 9, 0, 0, 10 },
					{ 1142, 4, 12, 0, 0, 6 },
					{ 1154, 8, 13, 0, -4, 10 },
					{ 1167, 8, 12, 0, 0, 10 },
					{ 1179, 2, 12, 0, 0, 4 },
					{ 1191, 3, 16, 0, -4, 5 },
					{ 1207, 8, 12, 0, 0, 10 },
					{ 1219, 2, 12, 0, 0, 4 },
					{ 1231, 12, 9, 0, 0, 14 },
					{ 1240, 8, 9, 0, 0, 10 },
					{ 1249, 8, 9, 0, 0, 10 },
					{ 1258, 8, 13, 0, -4, 10 },
					{ 1271, 8, 13, 0, -4, 10 },
					{ 1284, 5, 9, 0, 0, 7 },
					{ 1293, 7, 9, 0, 0, 9 },
					{ 1302, 4, 11, 0, 0, 6 },
					{ 1313, 8, 9, 0, 0, 10 },
					{ 1322, 8, 9, 0, 0, 10 },
					{ 1331, 12, 9, 0, 0, 14 },
					{ 1340, 7, 9, 0, 0, 9 },
					{ 1349, 8, 13, 0, -4, 10 },
					{ 1362, 7, 9, 0, 0, 9 },
					{ 1371, 4, 15, 0, -1, 6 },
					{ 1386, 1, 16, 0, 0, 3 },
					{ 1386, 4, 15, 0, -1, 6 },
					{ 1401, 8, 12, 0, 0, 10 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 0, 0, 0, 0, 0 },
					{ 1413, 33, 1, 0, 0, 32 },
					{ 1415, 33, 1, 0, 0, 32 },
					{ 1417, 0, 0, 0, 0, 0 },
					{ 1417, 33, 1, 0, 0, 32 },
					{ 1419, 33, 1, 0, 0, 32 },
					{ 1421, 0, 0, 0, 0, 0 },
					{ 1421, 33, 1, 0, 0, 32 },
					{ 1423, 33, 1, 0, 0, 32 },
					{ 1425, 33, 1, 0, 0, 32 },
					{ 1427, 0, 0, 0, 0, 0 },
					{ 1427, 0, 0, 0, 0, 0 },
					{ 1427, 0, 0, 0, 0, 0 },
					{ 1427, 0, 0, 0, 0, 0 },
					{ 1427, 0, 0, 0, 0, 0 },
					{ 1427, 0, 0, 0, 0, 0 },
					{ 1427, 0, 0, 0, 0, 0 },
					{ 1427, 0, 0, 0, 0, 0 },
					{ 1427, 0, 0, 0, 0, 0 },
					{ 1427, 0, 0, 0, 0, 0 },
					{ 1427, 0, 0, 0, 0, 0 },
					{ 1427, 0, 0, 0, 0, 0 }
				};
#define	FONT_NUMBER	2
#define	DEFINE_FONT	defrasterfont(FONT_NUMBER,30,256,chars,1427,raster_f);
#define	USE_FONT	font(FONT_NUMBER);
